<?php
/**
 * Plugin Name: Freepay.IR - Restrict Content Pro
 * Description: پلاگین پرداخت، شبکه پرداخت فری پی برای Restrict Content Pro
 * Plugin URI: https://freepay.ir
 * Author: تیم فنی فری پی
 * Author URI: https://freepay.ir
 * Version: 1.0
 **/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
require_once( 'freepay_session.php' );
if ( ! class_exists( 'RCP_freepay' ) ) {
	class RCP_freepay {
		public function __construct() {
			add_action( 'init', array( $this, 'freepay_Verify' ) );
			add_action( 'rcp_payments_settings', array( $this, 'freepay_Setting' ) );
			add_action( 'rcp_gateway_freepay', array( $this, 'freepay_Request' ) );
			add_filter( 'rcp_payment_gateways', array( $this, 'freepay_Register' ) );
			if ( ! function_exists( 'RCP_IRAN_Currencies_By_freepay' ) && ! function_exists( 'RCP_IRAN_Currencies' ) ) {
				add_filter( 'rcp_currencies', array( $this, 'RCP_IRAN_Currencies' ) );
			}
		}

		public function RCP_IRAN_Currencies( $currencies ) {
			unset( $currencies['RIAL'] );
			$currencies['تومان'] = __( 'تومان', 'rcp_freepay' );
			$currencies['ریال']  = __( 'ریال', 'rcp_freepay' );

			return $currencies;
		}

		public function freepay_Register( $gateways ) {
			global $rcp_options;

			if ( version_compare( RCP_PLUGIN_VERSION, '2.1.0', '<' ) ) {
				$gateways['freepay'] = isset( $rcp_options['freepay_name'] ) ? $rcp_options['freepay_name'] : __( 'شبکه پرداخت فری پی', 'rcp_freepay' );
			} else {
				$gateways['freepay'] = array(
					'label'       => isset( $rcp_options['freepay_name'] ) ? $rcp_options['freepay_name'] : __( 'شبکه پرداخت فری پی', 'rcp_freepay' ),
					'admin_label' => isset( $rcp_options['freepay_name'] ) ? $rcp_options['freepay_name'] : __( 'شبکه پرداخت فری پی', 'rcp_freepay' ),
				);
			}

			return $gateways;
		}

		public function freepay_Setting( $rcp_options ) {
			?>
            <hr/>
            <table class="form-table">
				<?php do_action( 'RCP_freepay_before_settings', $rcp_options ); ?>
                <tr valign="top">
                    <th colspan=2><h3><?php _e( 'تنظیمات شبکه پرداخت فری پی', 'rcp_freepay' ); ?></h3>
                    </th>
                </tr>
                <tr valign="top">
                    <th>
                        <label for="rcp_settings[freepay_api]"><?php _e( 'کد اعتبارسنجی', 'rcp_freepay' ); ?></label>
                    </th>
                    <td>
                        <input class="regular-text" id="rcp_settings[freepay_api]" style="width: 300px;"
                               name="rcp_settings[freepay_api]"
                               value="<?php if ( isset( $rcp_options['freepay_api'] ) ) {
							       echo $rcp_options['freepay_api'];
						       } ?>"/>
                    </td>
                </tr>
                <tr valign="top">
                    <th>
                        <label for="rcp_settings[freepay_query_name]"><?php _e( 'نام لاتین درگاه', 'rcp_freepay' ); ?></label>
                    </th>
                    <td>
                        <input class="regular-text" id="rcp_settings[freepay_query_name]" style="width: 300px;"
                               name="rcp_settings[freepay_query_name]"
                               value="<?php echo isset( $rcp_options['freepay_query_name'] ) ? $rcp_options['freepay_query_name'] : 'freepay'; ?>"/>
                        <div class="description"><?php _e( 'این نام در هنگام بازگشت از بانک در آدرس بازگشت از بانک نمایان خواهد شد . از به کاربردن حروف زائد و فاصله جدا خودداری نمایید .', 'rcp_freepay' ); ?></div>
                    </td>
                </tr>
                <tr valign="top">
                    <th>
                        <label for="rcp_settings[freepay_name]"><?php _e( 'نام نمایشی درگاه', 'rcp_freepay' ); ?></label>
                    </th>
                    <td>
                        <input class="regular-text" id="rcp_settings[freepay_name]" style="width: 300px;"
                               name="rcp_settings[freepay_name]"
                               value="<?php echo isset( $rcp_options['freepay_name'] ) ? $rcp_options['freepay_name'] : __( 'شبکه پرداخت فری پی', 'rcp_freepay' ); ?>"/>
                    </td>
                </tr>
                <tr valign="top">
                    <th>
                        <label><?php _e( 'تذکر ', 'rcp_freepay' ); ?></label>
                    </th>
                    <td>
                        <div class="description"><?php _e( 'از سربرگ مربوط به ثبت نام در تنظیمات افزونه حتما یک برگه برای بازگشت از بانک انتخاب نمایید . ترجیحا نامک برگه را لاتین قرار دهید .<br/> نیازی به قرار دادن شورت کد خاصی در برگه نیست و میتواند برگه ی خالی باشد .', 'rcp_freepay' ); ?></div>
                    </td>
                </tr>
				<?php do_action( 'RCP_freepay_after_settings', $rcp_options ); ?>
            </table>
			<?php

		}

		public function freepay_Request( $subscription_data ) {
			global $rcp_options;
			ob_start();
			$query                 = isset( $rcp_options['freepay_query_name'] ) ? $rcp_options['freepay_query_name'] : 'freepay';
			$amount                = str_replace( ',', '', $subscription_data['price'] );
			$freepay_payment_data = array(
				'user_id'           => $subscription_data['user_id'],
				'subscription_name' => $subscription_data['subscription_name'],
				'subscription_key'  => $subscription_data['key'],
				'amount'            => $amount
			);

			$_freepay_session = freepay_Session::get_instance();
			@session_start();
			$_freepay_session['freepay_payment_data'] = $freepay_payment_data;
			$_SESSION["freepay_payment_data"]          = $freepay_payment_data;
			do_action( 'RCP_Before_Sending_to_freepay', $subscription_data );
			if ( $rcp_options['currency'] == 'تومان' || $rcp_options['currency'] == 'TOMAN' || $rcp_options['currency'] == 'تومان ایران' || $rcp_options['currency'] == 'Iranian Toman (&#65020;)' ) {
				$amount = $amount * 10;
			}

			$Price      = intval( $amount );
			$ReturnPath = add_query_arg( 'gateway', $query, $subscription_data['return_url'] );

			$merchant_id  = $rcp_options['freepay_api'];
			$amount       = $Price;
			$callback_url = urlencode( $ReturnPath );
			$host=$_SERVER['SERVER_NAME'];
			$ch = curl_init();
			curl_setopt( $ch, CURLOPT_URL, 'http://freepay.ir/payment/request' );
			curl_setopt( $ch, CURLOPT_POSTFIELDS, "api=$merchant_id&amount=$amount&callback=$callback_url&host=$host" );
			curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
			curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
			$response = curl_exec( $ch );
			curl_close( $ch );
			$result = json_decode( $response );

			if ( $result->type == 'success' ) {
				ob_end_flush();
				ob_end_clean();
				$_SESSION['auth'] = $result->auth;
				$redirect_page = "https://freepay.ir/payment/gateway:".$result->auth;
				if ( ! headers_sent() ) {
					header( 'Location: ' . $redirect_page );
					exit;
				} else {
					echo "<script type='text/javascript'>window.onload = function () { top.location.href = '" . $redirect_page . "'; };</script>";
					exit;
				}
			} else {
				wp_die( sprintf( __( 'متاسفانه پرداخت به دلیل خطای زیر امکان پذیر نمی باشد . <br/><b> %s </b>', 'rcp_freepay' ), $this->Fault( $result->message ) ) );
			}
			exit();
		}

		public function freepay_Verify() {
			if ( ! isset( $_GET['gateway'] ) ) {
				return;
			}

			if ( ! class_exists( 'RCP_Payments' ) ) {
				return;
			}

			global $rcp_options, $wpdb, $rcp_payments_db_name;
			@session_start();
			$_freepay_session = freepay_Session::get_instance();
			if ( isset( $_freepay_session['freepay_payment_data'] ) ) {
				$freepay_payment_data = $_freepay_session['freepay_payment_data'];
			} else {
				$freepay_payment_data = isset( $_SESSION["freepay_payment_data"] ) ? $_SESSION["freepay_payment_data"] : '';
			}

			$query = isset( $rcp_options['freepay_query_name'] ) ? $rcp_options['freepay_query_name'] : 'freepay';

			if ( ( $_GET['gateway'] == $query ) && $freepay_payment_data ) {
				$user_id           = $freepay_payment_data['user_id'];
				$subscription_name = $freepay_payment_data['subscription_name'];
				$subscription_key  = $freepay_payment_data['subscription_key'];
				$amount            = $freepay_payment_data['amount'];

				$subscription_id = rcp_get_subscription_id( $user_id );
				$user_data       = get_userdata( $user_id );
				$payment_method  = isset( $rcp_options['freepay_name'] ) ? $rcp_options['freepay_name'] : __( 'شبکه پرداخت فری پی', 'rcp_freepay' );

				if ( ! $user_data || ! $subscription_id || ! rcp_get_subscription_details( $subscription_id ) ) {
					return;
				}

				$new_payment = 1;
				if ( $wpdb->get_results( $wpdb->prepare( "SELECT id FROM " . $rcp_payments_db_name . " WHERE `subscription_key`='%s' AND `payment_type`='%s';", $subscription_key, $payment_method ) ) ) {
					$new_payment = 0;
				}

				unset( $GLOBALS['freepay_new'] );
				$GLOBALS['freepay_new'] = $new_payment;
				global $new;
				$new = $new_payment;

				if ( $new_payment == 1 ) {
					$merchant_id = $rcp_options['freepay_api'];
					$Price       = intval( $amount );
					if ( $rcp_options['currency'] == 'تومان' || $rcp_options['currency'] == 'TOMAN' || $rcp_options['currency'] == 'تومان ایران' || $rcp_options['currency'] == 'Iranian Toman (&#65020;)' ) {
						$Price = $Price * 10;
					}
					$authority = (int) trim($_GET['authority']);
					if ( $authority ) {
						$ch = curl_init();
						curl_setopt( $ch, CURLOPT_URL, 'http://freepay.ir/payment/verify' );
						curl_setopt( $ch, CURLOPT_POSTFIELDS, "authority=$authority&api=$merchant_id" );
						curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, false );
						curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
						$response = curl_exec( $ch );
						curl_close( $ch );
						$result = json_decode( $response );
						if ( $result->status == 1 && $result->message == 'ok' && $_SESSION['auth'] == $authority ) {
							$payment_status = 'completed';
							$fault          = 0;
							$transaction_id = $authority;
						} else {
							$payment_status = 'failed';
							$fault          = $result->message;
							$transaction_id = 0;
						}
					} else {
						$payment_status = 'cancelled';
						$fault          = 0;
						$transaction_id = 0;
					}

					unset( $GLOBALS['freepay_payment_status'] );
					unset( $GLOBALS['freepay_transaction_id'] );
					unset( $GLOBALS['freepay_fault'] );
					unset( $GLOBALS['freepay_subscription_key'] );
					$GLOBALS['freepay_payment_status']   = $payment_status;
					$GLOBALS['freepay_transaction_id']   = $transaction_id;
					$GLOBALS['freepay_subscription_key'] = $subscription_key;
					$GLOBALS['freepay_fault']            = $fault;
					global $freepay_transaction;
					$freepay_transaction                              = array();
					$freepay_transaction['freepay_payment_status']   = $payment_status;
					$freepay_transaction['freepay_transaction_id']   = $transaction_id;
					$freepay_transaction['freepay_subscription_key'] = $subscription_key;
					$freepay_transaction['freepay_fault']            = $fault;

					if ( $payment_status == 'completed' ) {
						$payment_data = array(
							'date'             => date( 'Y-m-d g:i:s' ),
							'subscription'     => $subscription_name,
							'payment_type'     => $payment_method,
							'subscription_key' => $subscription_key,
							'amount'           => $amount,
							'user_id'          => $user_id,
							'transaction_id'   => $transaction_id
						);

						do_action( 'RCP_freepay_Insert_Payment', $payment_data, $user_id );

						$rcp_payments = new RCP_Payments();
						$rcp_payments->insert( $payment_data );

						rcp_set_status( $user_id, 'active' );

						if ( version_compare( RCP_PLUGIN_VERSION, '2.1.0', '<' ) ) {
							rcp_email_subscription_status( $user_id, 'active' );
							if ( ! isset( $rcp_options['disable_new_user_notices'] ) ) {
								wp_new_user_notification( $user_id );
							}
						}

						update_user_meta( $user_id, 'rcp_payment_profile_id', $user_id );
						update_user_meta( $user_id, 'rcp_signup_method', 'live' );
						update_user_meta( $user_id, 'rcp_recurring', 'no' );

						$subscription          = rcp_get_subscription_details( rcp_get_subscription_id( $user_id ) );
						$member_new_expiration = date( 'Y-m-d H:i:s', strtotime( '+' . $subscription->duration . ' ' . $subscription->duration_unit . ' 23:59:59' ) );
						rcp_set_expiration_date( $user_id, $member_new_expiration );
						delete_user_meta( $user_id, '_rcp_expired_email_sent' );

						$log_data = array(
							'post_title'   => __( 'تایید پرداخت', 'rcp_freepay' ),
							'post_content' => __( 'پرداخت با موفقیت انجام شد . کد تراکنش : ', 'rcp_freepay' ) . $transaction_id . __( ' .  روش پرداخت : ', 'rcp_freepay' ) . $payment_method,
							'post_parent'  => 0,
							'log_type'     => 'gateway_error'
						);

						$log_meta = array(
							'user_subscription' => $subscription_name,
							'user_id'           => $user_id
						);

						$log_entry = WP_Logging::insert_log( $log_data, $log_meta );

						do_action( 'RCP_freepay_Completed', $user_id );
					}

					if ( $payment_status == 'cancelled' ) {
						$log_data = array(
							'post_title'   => __( 'انصراف از پرداخت', 'rcp_freepay' ),
							'post_content' => __( 'تراکنش به دلیل انصراف کاربر از پرداخت ، ناتمام باقی ماند .', 'rcp_freepay' ) . __( ' روش پرداخت : ', 'rcp_freepay' ) . $payment_method,
							'post_parent'  => 0,
							'log_type'     => 'gateway_error'
						);

						$log_meta = array(
							'user_subscription' => $subscription_name,
							'user_id'           => $user_id
						);

						$log_entry = WP_Logging::insert_log( $log_data, $log_meta );

						//Action For freepay or RCP Developers...
						do_action( 'RCP_freepay_Cancelled', $user_id );
					}

					if ( $payment_status == 'failed' ) {
						$log_data = array(
							'post_title'   => __( 'خطا در پرداخت', 'rcp_freepay' ),
							'post_content' => __( 'تراکنش به دلیل خطای رو به رو ناموفق باقی ماند :', 'rcp_freepay' ) . $this->Fault( $fault ) . __( ' روش پرداخت : ', 'rcp_freepay' ) . $payment_method,
							'post_parent'  => 0,
							'log_type'     => 'gateway_error'
						);

						$log_meta = array(
							'user_subscription' => $subscription_name,
							'user_id'           => $user_id
						);

						$log_entry = WP_Logging::insert_log( $log_data, $log_meta );

						do_action( 'RCP_freepay_Failed', $user_id );
					}
				}
				add_filter( 'the_content', array( $this, 'freepay_Content_After_Return' ) );
			}
		}


		public function freepay_Content_After_Return( $content ) {
			global $freepay_transaction, $new;

			$_freepay_session = freepay_Session::get_instance();
			@session_start();

			$new_payment = isset( $GLOBALS['freepay_new'] ) ? $GLOBALS['freepay_new'] : $new;

			$payment_status = isset( $GLOBALS['freepay_payment_status'] ) ? $GLOBALS['freepay_payment_status'] : $freepay_transaction['freepay_payment_status'];
			$transaction_id = isset( $GLOBALS['freepay_transaction_id'] ) ? $GLOBALS['freepay_transaction_id'] : $freepay_transaction['freepay_transaction_id'];
			$fault          = isset( $GLOBALS['freepay_fault'] ) ? $this->Fault( $GLOBALS['freepay_fault'] ) : $this->Fault( $freepay_transaction['freepay_fault'] );

			if ( $new_payment == 1 ) {
				$freepay_data = array(
					'payment_status' => $payment_status,
					'transaction_id' => $transaction_id,
					'fault'          => $fault
				);

				$_freepay_session['freepay_data'] = $freepay_data;
				$_SESSION["freepay_data"]          = $freepay_data;
			} else {
				if ( isset( $_freepay_session['freepay_data'] ) ) {
					$freepay_payment_data = $_freepay_session['freepay_data'];
				} else {
					$freepay_payment_data = isset( $_SESSION["freepay_data"] ) ? $_SESSION["freepay_data"] : '';
				}

				$payment_status = isset( $freepay_payment_data['payment_status'] ) ? $freepay_payment_data['payment_status'] : '';
				$transaction_id = isset( $freepay_payment_data['transaction_id'] ) ? $freepay_payment_data['transaction_id'] : '';
				$fault          = isset( $freepay_payment_data['fault'] ) ? $this->Fault( $freepay_payment_data['fault'] ) : '';
			}

			$message = '';

			if ( $payment_status == 'completed' ) {
				$message = '<br/>' . __( 'پرداخت با موفقیت انجام شد . کد تراکنش : ', 'rcp_freepay' ) . $transaction_id . '<br/>';
			}

			if ( $payment_status == 'cancelled' ) {
				$message = '<br/>' . __( 'تراکنش به دلیل انصراف شما نا تمام باقی ماند .', 'rcp_freepay' );
			}

			if ( $payment_status == 'failed' ) {
				$message = '<br/>' . __( 'تراکنش به دلیل خطای زیر ناموفق باقی ماند :', 'rcp_freepay' ) . '<br/>' . $fault . '<br/>';
			}

			return $content . $message;
		}

		private static function Fault( $error ) {
			return $error;
		}
	}
}
new RCP_freepay();
if ( ! function_exists( 'change_cancelled_to_pending_By_freepay' ) ) {
	add_action( 'rcp_set_status', 'change_cancelled_to_pending_By_freepay', 10, 2 );
	function change_cancelled_to_pending_By_freepay( $status, $user_id ) {
		if ( 'cancelled' == $status ) {
			rcp_set_status( $user_id, 'expired' );
		}

		return true;
	}
}
?>